@@ -8,6 +8,7 @@ try:
 except ImportError:
     from urllib.parse import urlparse
 import sys
+import uuid
 
 from peewee import *
 from playhouse.db_url import connect
@@ -18,14 +19,15 @@ from playhouse.reflection import Introspector
 if sys.version_info[0] == 3:
     basestring = str
     from functools import reduce
-    def open_file(f, mode):
-        return open(f, mode, encoding='utf8')
+    def open_file(f, mode, encoding='utf8'):
+        return open(f, mode, encoding=encoding)
 else:
-    open_file = open
+    def open_file(f, mode, encoding='utf8'):
+        return open(f, mode)
 
 
 class DataSet(object):
-    def __init__(self, url, bare_fields=False):
+    def __init__(self, url, include_views=False, **kwargs):
         if isinstance(url, Database):
             self._url = None
             self._database = url
@@ -38,14 +40,17 @@ class DataSet(object):
             # Connect to the database.
             self._database = connect(url)
 
-        self._database.connect()
+        # Open a connection if one does not already exist.
+        self._database.connect(reuse_if_open=True)
 
         # Introspect the database and generate models.
         self._introspector = Introspector.from_database(self._database)
+        self._include_views = include_views
         self._models = self._introspector.generate_models(
             skip_invalid=True,
             literal_column_names=True,
-            bare_fields=bare_fields)
+            include_views=self._include_views,
+            **kwargs)
         self._migrator = SchemaMigrator.from_database(self._database)
 
         class BaseModel(Model):
@@ -77,13 +82,20 @@ class DataSet(object):
 
     @property
     def tables(self):
-        return self._database.get_tables()
+        tables = self._database.get_tables()
+        if self._include_views:
+            tables += self.views
+        return tables
+
+    @property
+    def views(self):
+        return [v.name for v in self._database.get_views()]
 
     def __contains__(self, table):
         return table in self.tables
 
-    def connect(self):
-        self._database.connect()
+    def connect(self, reuse_if_open=False):
+        self._database.connect(reuse_if_open=reuse_if_open)
 
     def close(self):
         self._database.close()
@@ -104,7 +116,8 @@ class DataSet(object):
         updated = self._introspector.generate_models(
             skip_invalid=True,
             table_names=dependencies,
-            literal_column_names=True)
+            literal_column_names=True,
+            include_views=self._include_views)
         self._models.update(updated)
 
     def get_table_dependencies(self, table):
@@ -128,14 +141,11 @@ class DataSet(object):
         if not self._database.is_closed():
             self.close()
 
-    def query(self, sql, params=None, commit=True):
-        return self._database.execute_sql(sql, params, commit)
+    def query(self, sql, params=None):
+        return self._database.execute_sql(sql, params)
 
     def transaction(self):
-        if self._database.transaction_depth() == 0:
-            return self._database.transaction()
-        else:
-            return self._database.savepoint()
+        return self._database.atomic()
 
     def _check_arguments(self, filename, file_obj, format, format_dict):
         if filename and file_obj:
@@ -150,10 +160,10 @@ class DataSet(object):
                 format, valid_formats))
 
     def freeze(self, query, format='csv', filename=None, file_obj=None,
-               **kwargs):
+               encoding='utf8', **kwargs):
         self._check_arguments(filename, file_obj, format, self._export_formats)
         if filename:
-            file_obj = open_file(filename, 'w')
+            file_obj = open_file(filename, 'w', encoding)
 
         exporter = self._export_formats[format](query)
         exporter.export(file_obj, **kwargs)
@@ -162,10 +172,10 @@ class DataSet(object):
             file_obj.close()
 
     def thaw(self, table, format='csv', filename=None, file_obj=None,
-             strict=False, **kwargs):
+             strict=False, encoding='utf8', **kwargs):
         self._check_arguments(filename, file_obj, format, self._export_formats)
         if filename:
-            file_obj = open_file(filename, 'r')
+            file_obj = open_file(filename, 'r', encoding)
 
         importer = self._import_formats[format](self[table], strict)
         count = importer.load(file_obj, **kwargs)
@@ -207,10 +217,9 @@ class Table(object):
             {'Meta': Meta})
 
     def create_index(self, columns, unique=False):
-        self.dataset._database.create_index(
-            self.model_class,
-            columns,
-            unique=unique)
+        index = ModelIndex(self.model_class, columns, unique=unique)
+        self.model_class.add_index(index)
+        self.dataset._database.execute(index)
 
     def _guess_field_type(self, value):
         if isinstance(value, basestring):
@@ -233,6 +242,7 @@ class Table(object):
 
     def _migrate_new_columns(self, data):
         new_keys = set(data) - set(self.model_class._meta.fields)
+        new_keys -= set(self.model_class._meta.columns)
         if new_keys:
             operations = []
             for key in new_keys:
@@ -339,12 +349,12 @@ class JSONExporter(Exporter):
             def default(o):
                 if isinstance(o, datetime_types):
                     return o.isoformat()
-                elif isinstance(o, Decimal):
+                elif isinstance(o, (Decimal, uuid.UUID)):
                     return str(o)
                 raise TypeError('Unable to serialize %r as JSON' % o)
         else:
             def default(o):
-                if isinstance(o, datetime_types + (Decimal,)):
+                if isinstance(o, datetime_types + (Decimal, uuid.UUID)):
                     return str(o)
                 raise TypeError('Unable to serialize %r as JSON' % o)
         return default
