@@ -3,6 +3,8 @@ import datetime
 import json
 import operator
 import os
+import sys
+import tempfile
 try:
     from StringIO import StringIO
 except ImportError:
@@ -12,9 +14,11 @@ from peewee import *
 from playhouse.dataset import DataSet
 from playhouse.dataset import Table
 
-from .base import db_loader
+from .base import IS_SQLITE_OLD
 from .base import ModelTestCase
 from .base import TestModel
+from .base import db_loader
+from .base import skip_if
 
 
 db = db_loader('sqlite')
@@ -27,6 +31,7 @@ class Note(TestModel):
     user = ForeignKeyField(User)
     content = TextField()
     timestamp = DateTimeField()
+    status = IntegerField(default=1)
 
 class Category(TestModel):
     name = TextField()
@@ -49,6 +54,14 @@ class TestDataSet(ModelTestCase):
         self.dataset.close()
         super(TestDataSet, self).tearDown()
 
+    def test_create_index(self):
+        users = self.dataset['users']
+        users.insert(username='u0')
+
+        users.create_index(['username'], True)
+        with self.assertRaises(IntegrityError):
+            users.insert(username='u0')
+
     def test_pass_database(self):
         db = SqliteDatabase(':memory:')
         dataset = DataSet(db)
@@ -58,6 +71,65 @@ class TestDataSet(ModelTestCase):
         users.insert(username='charlie')
         self.assertEqual(list(users), [{'id': 1, 'username': 'charlie'}])
 
+    @skip_if(IS_SQLITE_OLD)
+    def test_with_views(self):
+        self.dataset.query('CREATE VIEW notes_public AS '
+                           'SELECT content, timestamp FROM note '
+                           'WHERE status = 1 ORDER BY timestamp DESC')
+        try:
+            self.assertTrue('notes_public' in self.dataset.views)
+            self.assertFalse('notes_public' in self.dataset.tables)
+
+            users = self.dataset['user']
+            with self.dataset.transaction():
+                users.insert(username='charlie')
+                users.insert(username='huey')
+
+            notes = self.dataset['note']
+            for i, (ct, st) in enumerate([('n1', 1), ('n2', 2), ('n3', 1)]):
+                notes.insert(content=ct, status=st, user_id='charlie',
+                             timestamp=datetime.datetime(2022, 1, 1 + i))
+
+            self.assertFalse('notes_public' in self.dataset)
+
+            # Create a new dataset instance with views enabled.
+            dataset = DataSet(self.dataset._database, include_views=True)
+            self.assertTrue('notes_public' in dataset)
+            public = dataset['notes_public']
+            self.assertEqual(public.columns, ['content', 'timestamp'])
+            self.assertEqual(list(public), [
+                {'content': 'n3', 'timestamp': datetime.datetime(2022, 1, 3)},
+                {'content': 'n1', 'timestamp': datetime.datetime(2022, 1, 1)}])
+        finally:
+            self.dataset.query('DROP VIEW notes_public')
+
+    def test_item_apis(self):
+        dataset = DataSet('sqlite:///:memory:')
+        users = dataset['users']
+        users.insert(username='charlie')
+        self.assertEqual(list(users), [{'id': 1, 'username': 'charlie'}])
+
+        users[2] = {'username': 'huey', 'color': 'white'}
+        self.assertEqual(list(users), [
+            {'id': 1, 'username': 'charlie', 'color': None},
+            {'id': 2, 'username': 'huey', 'color': 'white'}])
+
+        users[2] = {'username': 'huey-x', 'kind': 'cat'}
+        self.assertEqual(list(users), [
+            {'id': 1, 'username': 'charlie', 'color': None, 'kind': None},
+            {'id': 2, 'username': 'huey-x', 'color': 'white', 'kind': 'cat'}])
+
+        del users[2]
+        self.assertEqual(list(users), [
+            {'id': 1, 'username': 'charlie', 'color': None, 'kind': None}])
+
+        users[1] = {'kind': 'person'}
+        users[2] = {'username': 'zaizee'}
+        users[2] = {'kind': 'cat'}
+        self.assertEqual(list(users), [
+            {'id': 1, 'username': 'charlie', 'color': None, 'kind': 'person'},
+            {'id': 2, 'username': 'zaizee', 'color': None, 'kind': 'cat'}])
+
     def create_users(self, n=2):
         user = self.dataset['user']
         for i in range(min(n, len(self.names))):
@@ -107,7 +179,8 @@ class TestDataSet(ModelTestCase):
 
         note = self.dataset['note']
         columns = sorted(note.columns)
-        self.assertEqual(columns, ['content', 'id', 'timestamp', 'user_id'])
+        self.assertEqual(columns, ['content', 'id', 'status', 'timestamp',
+                                   'user_id'])
 
         category = self.dataset['category']
         columns = sorted(category.columns)
@@ -122,6 +195,15 @@ class TestDataSet(ModelTestCase):
         self.assertEqual(sorted(Foo.columns), ['data', 'id'])
         self.assertTrue('foo' in self.dataset._models)
 
+        self.dataset._models['foo'].drop_table()
+        self.dataset.update_cache()
+        self.assertTrue('foo' not in self.database.get_tables())
+
+        # This will create the table again.
+        Foo = self.dataset['foo']
+        self.assertTrue('foo' in self.database.get_tables())
+        self.assertEqual(Foo.columns, ['id'])
+
     def assertQuery(self, query, expected, sort_key='id'):
         key = operator.itemgetter(sort_key)
         self.assertEqual(
@@ -238,17 +320,20 @@ class TestDataSet(ModelTestCase):
             note.insert(
                 content='note %s' % i,
                 timestamp=datetime.date(2014, 1, i),
+                status=i,
                 user_id='charlie')
 
         notes = sorted(note.all(), key=operator.itemgetter('id'))
         self.assertEqual(notes[0], {
             'content': 'note 1',
             'id': 1,
+            'status': 1,
             'timestamp': datetime.datetime(2014, 1, 1),
             'user_id': 'charlie'})
         self.assertEqual(notes[-1], {
             'content': 'note 3',
             'id': 3,
+            'status': 3,
             'timestamp': datetime.datetime(2014, 1, 3),
             'user_id': 'charlie'})
 
@@ -317,13 +402,40 @@ class TestDataSet(ModelTestCase):
             'charlie',
             'huey'])
 
+    @skip_if(sys.version_info[0] < 3, 'requires python 3.x')
+    def test_freeze_thaw_csv_utf8(self):
+        self._test_freeze_thaw_utf8('csv')
+
+    def test_freeze_thaw_json_utf8(self):
+        self._test_freeze_thaw_utf8('json')
+
+    def _test_freeze_thaw_utf8(self, fmt):
+        username_bytes = b'\xd0\x92obby'  # Bobby with cyrillic "B".
+        username_str = username_bytes.decode('utf8')
+        u = User.create(username=username_str)
+
+        # Freeze the data as a the given format.
+        user = self.dataset['user']
+        filename = tempfile.mktemp()  # Get a filename.
+        self.dataset.freeze(user.all(), fmt, filename)
+
+        # Clear out the table and reload.
+        User.delete().execute()
+        self.assertEqual(list(user.all()), [])
+
+        # Thaw the frozen data.
+        n = user.thaw(format=fmt, filename=filename)
+        self.assertEqual(n, 1)
+        self.assertEqual(list(user.all()), [{'username': username_str}])
+
     def test_freeze_thaw(self):
         user = self.dataset['user']
         user.insert(username='charlie')
 
         note = self.dataset['note']
         note_ts = datetime.datetime(2017, 1, 2, 3, 4, 5)
-        note.insert(content='foo', timestamp=note_ts, user_id='charlie')
+        note.insert(content='foo', timestamp=note_ts, user_id='charlie',
+                    status=2)
 
         buf = StringIO()
         self.dataset.freeze(note.all(), 'json', file_obj=buf)
@@ -331,6 +443,7 @@ class TestDataSet(ModelTestCase):
             'id': 1,
             'user_id': 'charlie',
             'content': 'foo',
+            'status': 2,
             'timestamp': '2017-01-02 03:04:05'}])
 
         note.delete(id=1)
@@ -342,6 +455,7 @@ class TestDataSet(ModelTestCase):
             'id': 1,
             'user_id': 'charlie',
             'content': 'foo',
+            'status': 2,
             'timestamp': note_ts}])
 
     def test_table_column_creation(self):
@@ -350,6 +464,18 @@ class TestDataSet(ModelTestCase):
         self.assertEqual(table.columns, ['id', 'name'])
         self.assertEqual(list(table.all()), [{'id': 1, 'name': 'charlie'}])
 
+    def test_table_column_creation_field_col(self):
+        table = self.dataset['people']
+        table.insert(**{'First Name': 'charlie'})
+        self.assertEqual(table.columns, ['id', 'First_Name'])
+        self.assertEqual(list(table.all()), [{'id': 1, 'First_Name': 'charlie'}])
+
+        table.insert(**{'First Name': 'huey'})
+        self.assertEqual(table.columns, ['id', 'First_Name'])
+        self.assertEqual(list(table.all().order_by(table.model_class.id)), [
+            {'id': 1, 'First_Name': 'charlie'},
+            {'id': 2, 'First_Name': 'huey'}])
+
     def test_import_json(self):
         table = self.dataset['people']
         table.insert(name='charlie')
