@@ -5,15 +5,23 @@ from functools import partial
 from peewee import *
 from playhouse.migrate import *
 from .base import BaseTestCase
+from .base import IS_CRDB
 from .base import IS_MYSQL
+from .base import IS_POSTGRESQL
+from .base import IS_PSYCOPG3
 from .base import IS_SQLITE
+from .base import IS_SQLITE_25
+from .base import IS_SQLITE_35
 from .base import ModelTestCase
 from .base import TestModel
 from .base import db
+from .base import get_in_memory_db
 from .base import requires_models
+from .base import requires_pglike
 from .base import requires_postgresql
 from .base import requires_sqlite
 from .base import skip_if
+from .base import skip_unless
 
 try:
     from psycopg2cffi import compat
@@ -81,7 +89,7 @@ class TestSchemaMigration(ModelTestCase):
         finally:
             self.database.close()
 
-    @requires_postgresql
+    @requires_pglike
     def test_add_table_constraint(self):
         price = FloatField(default=0.)
         migrate(self.migrator.add_column('tag', 'price', price),
@@ -119,7 +127,7 @@ class TestSchemaMigration(ModelTestCase):
         with self.database.atomic():
             self.assertRaises(IntegrityError, Tag2.create, tag='t2', alt_id=1)
 
-    @requires_postgresql
+    @requires_pglike
     def test_drop_table_constraint(self):
         price = FloatField(default=0.)
         migrate(
@@ -234,18 +242,19 @@ class TestSchemaMigration(ModelTestCase):
         cursor = self.database.execute_sql('select * from %s limit 1' % tbl)
         return set([col[0] for col in cursor.description])
 
-    def test_drop_column(self):
+    def test_drop_column(self, legacy=False):
+        kw = {'legacy': legacy} if IS_SQLITE else {}
         self._create_people()
         migrate(
-            self.migrator.drop_column('person', 'last_name'),
-            self.migrator.drop_column('person', 'dob'))
+            self.migrator.drop_column('person', 'last_name', **kw),
+            self.migrator.drop_column('person', 'dob', **kw))
 
         column_names = self.get_column_names('person')
         self.assertEqual(column_names, set(['id', 'first_name']))
 
         User.create(id='charlie', password='12345')
         User.create(id='huey', password='meow')
-        migrate(self.migrator.drop_column('users', 'password'))
+        migrate(self.migrator.drop_column('users', 'password', **kw))
 
         column_names = self.get_column_names('users')
         self.assertEqual(column_names, set(['id']))
@@ -254,11 +263,16 @@ class TestSchemaMigration(ModelTestCase):
             ('charlie',),
             ('huey',),])
 
-    def test_rename_column(self):
+    @skip_unless(IS_SQLITE_35, 'Requires sqlite 3.35 or newer')
+    def test_drop_column_sqlite_legacy(self):
+        self.test_drop_column(legacy=True)
+
+    def test_rename_column(self, legacy=False):
+        kw = {'legacy': legacy} if IS_SQLITE else {}
         self._create_people()
         migrate(
-            self.migrator.rename_column('person', 'first_name', 'first'),
-            self.migrator.rename_column('person', 'last_name', 'last'))
+            self.migrator.rename_column('person', 'first_name', 'first', **kw),
+            self.migrator.rename_column('person', 'last_name', 'last', **kw))
 
         column_names = self.get_column_names('person')
         self.assertEqual(column_names, set(['id', 'first', 'last', 'dob']))
@@ -280,14 +294,19 @@ class TestSchemaMigration(ModelTestCase):
                  .order_by(NewPerson.first))
         self.assertEqual(list(query.tuples()), self._person_data)
 
-    def test_rename_gh380(self):
+    @skip_unless(IS_SQLITE_25, 'Requires sqlite 3.25 or newer')
+    def test_rename_column_sqlite_legacy(self):
+        self.test_rename_column(legacy=True)
+
+    def test_rename_gh380(self, legacy=False):
+        kw = {'legacy': legacy} if IS_SQLITE else {}
         u1 = User.create(id='charlie')
         u2 = User.create(id='huey')
         p1 = Page.create(name='p1-1', user=u1)
         p2 = Page.create(name='p2-1', user=u1)
         p3 = Page.create(name='p3-2', user=u2)
 
-        migrate(self.migrator.rename_column('page', 'name', 'title'))
+        migrate(self.migrator.rename_column('page', 'name', 'title', **kw))
 
         column_names = self.get_column_names('page')
         self.assertEqual(column_names, set(['id', 'title', 'user_id']))
@@ -309,6 +328,34 @@ class TestSchemaMigration(ModelTestCase):
             [(np.title, np.user.id) for np in query],
             [('p1-1', 'charlie'), ('p2-1', 'charlie'), ('p3-2', 'huey')])
 
+    @skip_unless(IS_SQLITE_25, 'Requires sqlite 3.25 or newer')
+    def test_rename_gh380_sqlite_legacy(self):
+        self.test_rename_gh380(legacy=True)
+
+    @skip_if(IS_PSYCOPG3, 'Psycopg3 chokes on the default value.')
+    def test_add_default_drop_default(self):
+        with self.database.transaction():
+            migrate(self.migrator.add_column_default('person', 'first_name',
+                                                     default='x'))
+
+        p = Person.create(last_name='Last')
+        p_db = Person.get(Person.last_name == 'Last')
+        self.assertEqual(p_db.first_name, 'x')
+
+        with self.database.transaction():
+            migrate(self.migrator.drop_column_default('person', 'first_name'))
+
+        if IS_MYSQL:
+            # MySQL, even though the column is NOT NULL, does not seem to be
+            # enforcing the constraint(?).
+            Person.create(last_name='Last2')
+            p_db = Person.get(Person.last_name == 'Last2')
+            self.assertEqual(p_db.first_name, '')
+        else:
+            with self.assertRaises(IntegrityError):
+                with self.database.transaction():
+                    Person.create(last_name='Last2')
+
     def test_add_not_null(self):
         self._create_people()
 
@@ -319,7 +366,8 @@ class TestSchemaMigration(ModelTestCase):
         # We cannot make the `dob` field not null because there is currently
         # a null value there.
         if self._exception_add_not_null:
-            self.assertRaises(IntegrityError, addNotNull)
+            with self.assertRaisesCtx((IntegrityError, InternalError)):
+                addNotNull()
 
         (Person
          .update(dob=datetime.date(2000, 1, 2))
@@ -361,7 +409,8 @@ class TestSchemaMigration(ModelTestCase):
                 migrate(self.migrator.add_not_null('page', 'user_id'))
 
         if self._exception_add_not_null:
-            self.assertRaises(IntegrityError, addNotNull)
+            with self.assertRaisesCtx((IntegrityError, InternalError)):
+                addNotNull()
 
         with self.database.transaction():
             Page.update(user=user).where(Page.user.is_null()).execute()
@@ -411,6 +460,8 @@ class TestSchemaMigration(ModelTestCase):
                 Tag.create,
                 tag='t3')
 
+        self.database.execute_sql('drop table tag_asdf')
+
     def test_add_index(self):
         # Create a unique index on first and last names.
         columns = ('first_name', 'last_name')
@@ -418,11 +469,8 @@ class TestSchemaMigration(ModelTestCase):
 
         Person.create(first_name='first', last_name='last')
         with self.database.transaction():
-            self.assertRaises(
-                IntegrityError,
-                Person.create,
-                first_name='first',
-                last_name='last')
+            with self.assertRaisesCtx((IntegrityError, InternalError)):
+                Person.create(first_name='first', last_name='last')
 
     def test_add_unique_column(self):
         uf = CharField(default='', unique=True)
@@ -522,6 +570,8 @@ class TestSchemaMigration(ModelTestCase):
                 last='Leifer',
                 field_default='bazer')
 
+        self.database.execute_sql('drop table person_nugg;')
+
     def test_add_foreign_key(self):
         if hasattr(Person, 'newtag_set'):
             delattr(Person, 'newtag_set')
@@ -556,7 +606,8 @@ class TestSchemaMigration(ModelTestCase):
         self.assertEqual(foreign_key.dest_table, 'person')
 
     def test_drop_foreign_key(self):
-        migrate(self.migrator.drop_column('page', 'user_id'))
+        kw = {'legacy': True} if IS_SQLITE else {}
+        migrate(self.migrator.drop_column('page', 'user_id', **kw))
         columns = self.database.get_columns('page')
         self.assertEqual(
             sorted(column.name for column in columns),
@@ -591,7 +642,7 @@ class TestSchemaMigration(ModelTestCase):
         self.assertEqual(foreign_key.dest_column, 'id')
         self.assertEqual(foreign_key.dest_table, 'users')
 
-    @requires_postgresql
+    @requires_pglike
     @requires_models(Tag)
     def test_add_column_with_index_type(self):
         from playhouse.postgres_ext import BinaryJSONField
@@ -605,22 +656,54 @@ class TestSchemaMigration(ModelTestCase):
              []),
         ])
 
+    @skip_if(IS_CRDB, 'crdb is still finnicky about changing types.')
+    def test_alter_column_type(self):
+        # Convert varchar to text.
+        field = TextField()
+        migrate(self.migrator.alter_column_type('tag', 'tag', field))
+        _, tag = self.database.get_columns('tag')
+        # name, type, null?, primary-key?, table, default.
+        data_type = 'TEXT' if IS_SQLITE else 'text'
+        self.assertEqual(tag, ('tag', data_type, False, False, 'tag', None))
+
+        # Convert date to datetime.
+        field = DateTimeField()
+        migrate(self.migrator.alter_column_type('person', 'dob', field))
+        _, _, _, dob = self.database.get_columns('person')
+        if IS_POSTGRESQL or IS_CRDB:
+            self.assertTrue(dob.data_type.startswith('timestamp'))
+        else:
+            self.assertEqual(dob.data_type.lower(), 'datetime')
+
+        # Convert text to integer.
+        field = IntegerField()
+        cast = '(tag::integer)' if IS_POSTGRESQL or IS_CRDB else None
+        migrate(self.migrator.alter_column_type('tag', 'tag', field, cast))
+        _, tag = self.database.get_columns('tag')
+        if IS_SQLITE:
+            data_type = 'INTEGER'
+        elif IS_MYSQL:
+            data_type = 'int'
+        else:
+            data_type = 'integer'
+        self.assertEqual(tag, ('tag', data_type, False, False, 'tag', None))
+
     @requires_sqlite
     def test_valid_column_required(self):
         self.assertRaises(
-            ValueError,
+            (OperationalError, ValueError),
             migrate,
             self.migrator.drop_column('page', 'column_does_not_exist'))
 
         self.assertRaises(
-            ValueError,
+            (OperationalError, ValueError),
             migrate,
             self.migrator.rename_column('page', 'xx', 'yy'))
 
     @requires_sqlite
     @requires_models(IndexModel)
     def test_table_case_insensitive(self):
-        migrate(self.migrator.drop_column('PaGe', 'name'))
+        migrate(self.migrator.drop_column('PaGe', 'name', legacy=True))
         column_names = self.get_column_names('page')
         self.assertEqual(column_names, set(['id', 'user_id']))
 
@@ -629,7 +712,8 @@ class TestSchemaMigration(ModelTestCase):
         column_names = self.get_column_names('page')
         self.assertEqual(column_names, set(['id', 'user_id', 'testing']))
 
-        migrate(self.migrator.drop_column('indeX_mOdel', 'first_name'))
+        migrate(self.migrator.drop_column('indeX_mOdel', 'first_name',
+                                          legacy=True))
         indexes = self.migrator.database.get_indexes('index_model')
         self.assertEqual(len(indexes), 1)
         self.assertEqual(indexes[0].name, 'index_model_data')
@@ -692,6 +776,8 @@ class TestSchemaMigration(ModelTestCase):
         self.assertEqual(queries, [
             ('ALTER TABLE "category" ADD COLUMN "parent_id" '
              'INTEGER REFERENCES "category" ("id") ON DELETE SET NULL', []),
+            ('CREATE INDEX "category_parent_id" ON "category" ("parent_id")',
+             []),
         ])
 
     @requires_sqlite
@@ -701,7 +787,8 @@ class TestSchemaMigration(ModelTestCase):
         migrate(self.migrator.rename_column(
             'index_model',
             'first_name',
-            'first'))
+            'first',
+            legacy=True))
 
         queries = [x.msg for x in self.history]
         self.assertEqual(queries, [
@@ -760,7 +847,7 @@ class TestSchemaMigration(ModelTestCase):
         self.reset_sql_history()
         new_fk = ForeignKeyField(User, User.id, null=True, on_delete='CASCADE')
         migrate(
-            self.migrator.drop_column('page', 'user_id'),
+            self.migrator.drop_column('page', 'user_id', legacy=True),
             self.migrator.add_column('page', 'user_id', new_fk))
 
         queries = [x.msg for x in self.history]
@@ -797,6 +884,7 @@ class TestSchemaMigration(ModelTestCase):
             # Add new foreign-key field with appropriate constraint.
             ('ALTER TABLE "page" ADD COLUMN "user_id" VARCHAR(20) '
              'REFERENCES "users" ("id") ON DELETE CASCADE', []),
+            ('CREATE INDEX "page_user_id" ON "page" ("user_id")', []),
         ])
 
         self.database.pragma('foreign_keys', 1)
@@ -820,3 +908,84 @@ class TestSchemaMigration(ModelTestCase):
         ]
         name = make_index_name('very_long_table_name', columns)
         self.assertEqual(len(name), 64)
+
+
+class BadNames(TestModel):
+    primary_data = TextField()
+    foreign_data = TextField()
+    data = TextField()
+
+    class Meta:
+        constraints = [
+            SQL('CONSTRAINT const1 UNIQUE (primary_data)'),
+            SQL('CONSTRAINT const2 UNIQUE (foreign_data)')]
+
+
+class HasChecks(TestModel):
+    key = TextField()
+    value = IntegerField()
+    class Meta:
+        constraints = [
+            SQL("CHECK (key != '')"),
+            SQL('CHECK (value > 0)')]
+
+
+class TestSqliteColumnNameRegression(ModelTestCase):
+    database = get_in_memory_db()
+    requires = [BadNames, HasChecks]
+
+    def test_sqlite_check_constraints(self):
+        HasChecks.create(key='k1', value=1)
+
+        migrator = SchemaMigrator.from_database(self.database)
+        extra = TextField(default='')
+        migrate(migrator.add_column('has_checks', 'extra', extra))
+
+        columns = self.database.get_columns('has_checks')
+        self.assertEqual([c.name for c in columns],
+                         ['id', 'key', 'value', 'extra'])
+
+        HC = Table('has_checks', ('id', 'key', 'value', 'extra'))
+        HC = HC.bind(self.database)
+
+        # Sanity-check: ensure we can create a new row.
+        data = {'key': 'k2', 'value': 2, 'extra': 'x2'}
+        self.assertTrue(HC.insert(data).execute())
+
+        # Check constraints preserved.
+        data = {'key': 'k0', 'value': 0, 'extra': 'x0'}
+        self.assertRaises(IntegrityError, HC.insert(data).execute)
+
+        data = {'key': '', 'value': 3, 'extra': 'x3'}
+        self.assertRaises(IntegrityError, HC.insert(data).execute)
+
+    def test_sqlite_column_name_constraint_regression(self):
+        BadNames.create(primary_data='pd', foreign_data='fd', data='d')
+
+        migrator = SchemaMigrator.from_database(self.database)
+        new_data = TextField(default='foo')
+        migrate(migrator.add_column('bad_names', 'new_data', new_data),
+                migrator.drop_column('bad_names', 'data'))
+
+        columns = self.database.get_columns('bad_names')
+        column_names = [column.name for column in columns]
+        self.assertEqual(column_names, ['id', 'primary_data', 'foreign_data',
+                                        'new_data'])
+
+        BNT = Table('bad_names', ('id', 'primary_data', 'foreign_data',
+                                  'new_data')).bind(self.database)
+        self.assertEqual([row for row in BNT.select()], [{
+            'id': 1,
+            'primary_data': 'pd',
+            'foreign_data': 'fd',
+            'new_data': 'foo'}])
+
+        # Verify constraints were carried over.
+        data = {'primary_data': 'pd', 'foreign_data': 'xx', 'new_data': 'd'}
+        self.assertRaises(IntegrityError, BNT.insert(data).execute)
+
+        data.update(primary_data='px', foreign_data='fd')
+        self.assertRaises(IntegrityError, BNT.insert(data).execute)
+
+        data.update(foreign_data='fx')
+        self.assertTrue(BNT.insert(data).execute())
