@@ -1,6 +1,7 @@
 import datetime
 
 from peewee import *
+from peewee import Alias
 from peewee import Database
 from peewee import ModelIndex
 
@@ -23,7 +24,7 @@ class CKM(TestModel):
 
 class TestModelSQL(ModelDatabaseTestCase):
     database = get_in_memory_db()
-    requires = [Category, CKM, Note, Person, Relationship, Sample, User]
+    requires = [Category, CKM, Note, Person, Relationship, Sample, User, DfltM]
 
     def test_select(self):
         query = (Person
@@ -59,6 +60,39 @@ class TestModelSQL(ModelDatabaseTestCase):
         query = query2.select(Category.name)
         self.assertSQL(query, 'SELECT "t1"."name" FROM "category" AS "t1"', [])
 
+    def test_select_extend(self):
+        query = Note.select()
+        ext = query.join(Person).select_extend(Person)
+        self.assertSQL(ext, (
+            'SELECT "t1"."id", "t1"."author_id", "t1"."content", "t2"."id", '
+            '"t2"."first", "t2"."last", "t2"."dob" '
+            'FROM "note" AS "t1" INNER JOIN "person" AS "t2" '
+            'ON ("t1"."author_id" = "t2"."id")'), [])
+
+    def test_selected_columns(self):
+        query = (Person
+                 .select(
+                     Person.first,
+                     Person.last,
+                     fn.COUNT(Note.id).alias('ct'))
+                 .join(Note))
+        f_first, f_last, f_ct = query.selected_columns
+        self.assertEqual(f_first.name, 'first')
+        self.assertTrue(f_first.model is Person)
+        self.assertEqual(f_last.name, 'last')
+        self.assertTrue(f_last.model is Person)
+        self.assertTrue(isinstance(f_ct, Alias))
+        f_ct = f_ct.unwrap()
+        self.assertEqual(f_ct.name, 'COUNT')
+        f_nid, = f_ct.arguments
+        self.assertEqual(f_nid.name, 'id')
+        self.assertTrue(f_nid.model is Note)
+
+        query.selected_columns = (Person.first,)
+        f_first, = query.selected_columns
+        self.assertEqual(f_first.name, 'first')
+        self.assertTrue(f_first.model is Person)
+
     def test_where_coerce(self):
         query = Person.select(Person.last).where(Person.id == '1337')
         self.assertSQL(query, (
@@ -171,6 +205,19 @@ class TestModelSQL(ModelDatabaseTestCase):
             'INNER JOIN "favorite" AS "t3" ON ("t3"."tweet_id" = "t1"."id")'),
             [])
 
+        query = Tweet.select(Tweet.id).left_outer_join(Favorite).switch(Tweet).left_outer_join(User)
+        self.assertSQL(query, (
+            'SELECT "t1"."id" FROM "tweet" AS "t1" '
+            'LEFT OUTER JOIN "favorite" AS "t2" ON ("t2"."tweet_id" = "t1"."id") '
+            'LEFT OUTER JOIN "users" AS "t3" ON ("t1"."user_id" = "t3"."id")'), [])
+
+        query = Tweet.select(Tweet.id).left_outer_join(User).switch(Tweet).left_outer_join(Favorite)
+        self.assertSQL(query, (
+            'SELECT "t1"."id" FROM "tweet" AS "t1" '
+            'LEFT OUTER JOIN "users" AS "t2" ON ("t1"."user_id" = "t2"."id") '
+            'LEFT OUTER JOIN "favorite" AS "t3" ON ("t3"."tweet_id" = "t1"."id")'),
+            [])
+
     def test_model_alias(self):
         TA = Tweet.alias()
         query = (User
@@ -206,6 +253,18 @@ class TestModelSQL(ModelDatabaseTestCase):
             'WHERE ((("t1"."id" >= ?) AND ("t1"."id" < ?)) AND '
             '("t1"."username" = ?))'), [1, 5, 'huey'])
 
+        query = User.filter(~DQ(id=1), username__in=('foo', 'bar'))
+        self.assertSQL(query, (
+            'SELECT "t1"."id", "t1"."username" FROM "users" AS "t1" '
+            'WHERE (NOT ("t1"."id" = ?) AND ("t1"."username" IN (?, ?)))'),
+            [1, 'foo', 'bar'])
+
+        query = User.filter((DQ(id=1) | DQ(id=2)), username__in=('foo', 'bar'))
+        self.assertSQL(query, (
+            'SELECT "t1"."id", "t1"."username" FROM "users" AS "t1" '
+            'WHERE ((("t1"."id" = ?) OR ("t1"."id" = ?)) AND '
+            '("t1"."username" IN (?, ?)))'), [1, 2, 'foo', 'bar'])
+
     def test_filter_expressions(self):
         query = User.filter(
             DQ(username__in=['huey', 'zaizee']) |
@@ -428,6 +487,21 @@ class TestModelSQL(ModelDatabaseTestCase):
             'INSERT INTO "sample" ("counter", "value") VALUES (?, ?), (?, ?)'),
             [3, 1., 2, 2.])
 
+    def test_insert_many_defaults_nulls(self):
+        data = [
+            {'name': 'd1'},
+            {'name': 'd2', 'dflt1': 10},
+            {'name': 'd3', 'dflt2': 30},
+            {'name': 'd4', 'dfltn': 40}]
+        fields = [DfltM.name, DfltM.dflt1, DfltM.dflt2, DfltM.dfltn]
+        self.assertSQL(DfltM.insert_many(data, fields=fields), (
+            'INSERT INTO "dflt_m" ("name", "dflt1", "dflt2", "dfltn") VALUES '
+            '(?, ?, ?, ?), (?, ?, ?, ?), (?, ?, ?, ?), (?, ?, ?, ?)'),
+            ['d1', 1, 2, None,
+             'd2', 10, 2, None,
+             'd3', 1, 30, None,
+             'd4', 1, 2, 40])
+
     def test_insert_many_list_with_fields(self):
         data = [(i,) for i in ('charlie', 'huey', 'zaizee')]
         query = User.insert_many(data, fields=[User.username])
@@ -843,6 +917,21 @@ class TestOnConflictSQL(ModelDatabaseTestCase):
             'DO UPDATE SET "b" = ("oc_test"."b" + ?) '
             'RETURNING "oc_test"."id"'), ['foo', 1, 0, 2])
 
+    def test_on_conflict_do_nothing(self):
+        query = OCTest.insert(a='foo', b=1).on_conflict(action='IGNORE')
+        self.assertSQL(query, (
+            'INSERT INTO "oc_test" ("a", "b", "c") VALUES (?, ?, ?) '
+            'ON CONFLICT DO NOTHING '
+            'RETURNING "oc_test"."id"'), ['foo', 1, 0])
+
+        query = OCTest.insert(a='foo', b=1).on_conflict(
+            conflict_target=(OCTest.a,),
+            action='IGNORE')
+        self.assertSQL(query, (
+            'INSERT INTO "oc_test" ("a", "b", "c") VALUES (?, ?, ?) '
+            'ON CONFLICT ("a") DO NOTHING '
+            'RETURNING "oc_test"."id"'), ['foo', 1, 0])
+
     def test_update_where_clause(self):
         # Add a new row with the given "a" value. If a conflict occurs,
         # re-insert with b=b+2 so long as the original b < 3.
