@@ -1,10 +1,12 @@
 import datetime
 import os
 import re
+import warnings
 
 from peewee import *
 from playhouse.reflection import *
 
+from .base import IS_CRDB
 from .base import IS_SQLITE_OLD
 from .base import ModelTestCase
 from .base import TestModel
@@ -67,6 +69,12 @@ class Nugget(TestModel):
     category = CharField()
 
 
+class NoPK(TestModel):
+    data = CharField()
+    class Meta:
+        primary_key = False
+
+
 class BaseReflectionTestCase(ModelTestCase):
     def setUp(self):
         super(BaseReflectionTestCase, self).setUp()
@@ -175,9 +183,28 @@ class TestReflection(BaseReflectionTestCase):
             ))
 
     def test_make_column_name(self):
+        # Tests for is_foreign_key=False.
         tests = (
             ('Column', 'column'),
-            ('Foo_iD', 'foo'),
+            ('Foo_id', 'foo_id'),
+            ('foo_id', 'foo_id'),
+            ('foo_id_id', 'foo_id_id'),
+            ('foo', 'foo'),
+            ('_id', '_id'),
+            ('a123', 'a123'),
+            ('and', 'and_'),
+            ('Class', 'class_'),
+            ('Class_ID', 'class_id'),
+            ('camelCase', 'camel_case'),
+            ('ABCdefGhi', 'ab_cdef_ghi'),
+        )
+        for col_name, expected in tests:
+            self.assertEqual(
+                self.introspector.make_column_name(col_name), expected)
+
+        # Tests for is_foreign_key=True.
+        tests = (
+            ('Foo_id', 'foo'),
             ('foo_id', 'foo'),
             ('foo_id_id', 'foo_id'),
             ('foo', 'foo'),
@@ -186,10 +213,12 @@ class TestReflection(BaseReflectionTestCase):
             ('and', 'and_'),
             ('Class', 'class_'),
             ('Class_ID', 'class_'),
+            ('camelCase', 'camel_case'),
+            ('ABCdefGhi', 'ab_cdef_ghi'),
         )
         for col_name, expected in tests:
             self.assertEqual(
-                self.introspector.make_column_name(col_name), expected)
+                self.introspector.make_column_name(col_name, True), expected)
 
     def test_make_model_name(self):
         tests = (
@@ -421,6 +450,20 @@ class TestReflection(BaseReflectionTestCase):
                                 '%s not in %s' % (actual, fields))
 
 
+class TestReflectNoPK(BaseReflectionTestCase):
+    requires = [NoPK]
+
+    def test_no_pk(self):
+        models = self.introspector.generate_models()
+        NoPK = models['no_pk']
+        if IS_CRDB:
+            # CockroachDB always includes a "rowid".
+            self.assertEqual(NoPK._meta.sorted_field_names, ['rowid', 'data'])
+        else:
+            self.assertEqual(NoPK._meta.sorted_field_names, ['data'])
+            self.assertTrue(NoPK._meta.primary_key is False)
+
+
 class EventLog(TestModel):
     data = CharField(constraints=[SQL('DEFAULT \'\'')])
     timestamp = DateTimeField(constraints=[SQL('DEFAULT current_timestamp')])
@@ -539,6 +582,7 @@ class TestReflectViews(BaseReflectionTestCase):
         self.assertTrue(isinstance(NotesPublic.timestamp, DateTimeField))
 
     @skip_if(IS_SQLITE_OLD)
+    @skip_if(IS_CRDB, 'crdb does not respect order by in view def')
     def test_introspect_view_integration(self):
         for i, (ct, st) in enumerate([('n1', 1), ('n2', 2), ('n3', 1)]):
             Note.create(content=ct, status=st,
@@ -549,3 +593,65 @@ class TestReflectViews(BaseReflectionTestCase):
         self.assertEqual([(np.content, np.timestamp) for np in NP.select()], [
             ('n3', datetime.datetime(2018, 1, 3)),
             ('n1', datetime.datetime(2018, 1, 1))])
+
+
+class TestCyclicalFK(BaseReflectionTestCase):
+    def setUp(self):
+        super(TestCyclicalFK, self).setUp()
+        warnings.filterwarnings('ignore')
+
+    @requires_sqlite
+    def test_cyclical_fk(self):
+        # NOTE: this schema was provided by a user.
+        cursor = self.database.cursor()
+        cursor.executescript(
+            'CREATE TABLE flow_run_state (id CHAR(36) NOT NULL, '
+            'flow_run_id CHAR(36) NOT NULL, '
+	    'CONSTRAINT pk_flow_run_state PRIMARY KEY (id), '
+	    'CONSTRAINT fk_flow_run_state__flow_run_id__flow_run '
+            'FOREIGN KEY(flow_run_id) REFERENCES flow_run (id) '
+            'ON DELETE cascade); '
+            'CREATE TABLE flow_run (id CHAR(36) NOT NULL, '
+	    'state_id CHAR(36) NOT NULL, '
+	    'CONSTRAINT pk_flow_run PRIMARY KEY (id), '
+	    'CONSTRAINT fk_flow_run__state_id__flow_run_state '
+            'FOREIGN KEY(state_id) REFERENCES flow_run_state (id) '
+            'ON DELETE SET NULL);')
+        M = self.introspector.generate_models()
+        FRS = M['flow_run_state']
+        FR = M['flow_run']
+        self.assertEqual(sorted(FR._meta.fields), ['id', 'state'])
+        self.assertEqual(sorted(FRS._meta.fields), ['flow_run', 'id'])
+        self.assertTrue(isinstance(FR.id, CharField))
+        self.assertTrue(isinstance(FR.state, ForeignKeyField))
+        self.assertTrue(FR.state.rel_model is FRS)
+        self.assertTrue(isinstance(FRS.id, CharField))
+        self.assertTrue(isinstance(FRS.flow_run, ForeignKeyField))
+        self.assertTrue(FRS.flow_run.rel_model is FR)
+
+
+class Event(TestModel):
+    key = TextField()
+    timestamp = DateTimeField(index=True)
+    metadata = TextField(default='')
+
+
+class TestInteractiveHelpers(ModelTestCase):
+    requires = [Category, Event]
+
+    def test_generate_models(self):
+        M = generate_models(self.database)
+        self.assertTrue('category' in M)
+        self.assertTrue('event' in M)
+
+        def assertFields(m, expected):
+            actual = [(f.name, f.field_type) for f in m._meta.sorted_fields]
+            self.assertEqual(actual, expected)
+
+        assertFields(M['category'], [('id', 'AUTO'), ('name', 'VARCHAR'),
+                                     ('parent', 'INT')])
+        assertFields(M['event'], [
+            ('id', 'AUTO'),
+            ('key', 'TEXT'),
+            ('timestamp', 'DATETIME'),
+            ('metadata', 'TEXT')])
